# encoding=utf-8
"""
mwtemplates
Copyright (c) 2012-2014 Dan Michael O. Heggø

Simple wikitext template parser and editor
"""

from mwtemplates.templateeditor2 import TemplateEditor
from mwtemplates.preprocessor import preprocessToXml



# encoding=utf-8
from __future__ import unicode_literals

# /Users/danmichael/code/mediawiki/core/includes/parser/Preprocessor_DOM.php
import re
import logging

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

# First we define a few useful functions from PHP:


def strspn(str1, str2, start=0, length=-1):
    str2 = re.escape(str2)
    if length == -1:
        m = re.match(r'^([' + str2 + ']*)', str1[start:])
    else:
        m = re.match(r'^([' + str2 + ']{,' + str(length) + '})', str1[start:])
    if m:
        return len(m.group(1))
    else:
        return 0


def strcspn(str1, str2, start=0, length=-1, debug=False):
    str2 = re.escape(str2)
    if length == -1:
        m = re.match(r'^([^' + str2 + ']*)', str1[start:])
    else:
        m = re.match(r'^([^' + str2 + ']{,' + str(length) + '})', str1[start:])
    if m:
        return len(m.group(1))
    else:
        return 0


def htmlspecialchars(text):
    return text.replace("&", "&amp;") \
               .replace('"', "&quot;") \
               .replace("<", "&lt;") \
               .replace(">", "&gt;")


class PPDStack(object):

    def __init__(self):
        self.stack = []
        self.rootAccum = ''

    @property
    def top(self):
        if len(self.stack) == 0:
            return False
        else:
            return self.stack[-1]

    @property
    def accum(self):
        if self.top is False:
            return self.rootAccum
        else:
            return self.top.accum

    @accum.setter
    def accum(self, value):
        if self.top is False:
            self.rootAccum = value
        else:
            self.top.accum = value

    def count(self):
        return len(self.stack)

    def getCurrentPart(self):
        if self.top is False:
            return False
        else:
            return self.top.getCurrentPart()

    def append(self, data):
        if type(data) == PPDStackElement:
            self.stack.append(data)
        else:
            self.stack.append(PPDStackElement(data))
        logger.debug('+ Adding to stack: %s...',
                     self.top.data['open'] * self.top.data['count'])

    def pop(self):
        logger.debug('- Popping from stack: ...%s',
                     self.top.data['close'] * self.top.data['count'])
        return self.stack.pop()

    def addPart(self, s=''):
        self.top.addPart(s)

    def getFlags(self):
        if len(self.stack) == 0:
            return {'findEquals': False, 'findPipe': False, 'inHeading': False}
        else:
            return self.top.getFlags()


class PPDStackElement(object):

    def __init__(self, data):
        self.data = data
        self.parts = [PPDPart()]

    @property
    def accum(self):
        return self.parts[-1].out

    @accum.setter
    def accum(self, value):
        self.parts[-1].out = value

    def addPart(self, s=''):
        self.parts.append(PPDPart(s))

    def getCurrentPart(self):
        return self.parts[-1]

    def getFlags(self):
        partCount = len(self.parts)
        findPipe = self.data['open'] != '\n' and self.data['open'] != '['
        return {
            'findPipe': findPipe,
            'findEquals': findPipe and partCount > 1 and 'eqpos' not in self.parts[-1],
            'inHeading': self.data['open'] == r'\n'
        }

    # Get the output string that would result if the close is not found.
    def breakSyntax(self, openingCount=False):
        if self.data['open'] == '\n':
            s = self.parts[0]
        else:
            if openingCount is False:
                openingCount = self.data['count']
            s = self.data['open'] * openingCount
            first = True
            for part in self.parts:
                if first:
                    first = False
                else:
                    s += '|'
                s += part.out
        return s


class PPDPart(dict):
    # Optional member variables:
    #   eqpos       Position of equals sign in output accumulator
    #   commentEnd  Past-the-end input pointer for the last comment encountered
    #   visualEnd   Past-the-end input pointer for the end of the accumulator
    #               minus comments

    def __init__(self, out=''):
        self['out'] = out

    def __getattr__(self, attr):
        return self[attr]

    def __setattr__(self, attr, value):
        self[attr] = value


def preprocessToXml(text):

    if type(text).__name__ != 'unicode' and type(text).__name__ != 'str':
        raise StandardError("preprocessToXml only accepts unicode or str")

    rules = {
        '{': {
            'end': '}',
            'names': {
                2: 'template',
                3: 'tplarg',
            },
            'min': 2,
            'max': 3
        },
        '[': {
            'end': ']',
            'names': {
                2: None
            },
            'min': 2,
            'max': 2
        }
    }

    # tja..
    xmlishRegex = '|'.join(['math', 'nowiki', 'pre'])

    # Use "A" modifier (anchored) instead of "^", because ^ doesn't work
    # with an offset
    # DM: A is not available in Python, so let's try ^
    elementsRegex = re.compile(r'^(%s)(?:\s|\/>|>)|(!--)' % xmlishRegex,
                               flags=re.I)
    # r'~(%s)(?:\s|\/>|>)|(!--)~iA' % xmlishRegex

    stack = PPDStack()
    searchBase = '[{<'     # removed \n for now
    lengthText = len(text)

    stack.accum = '<root>'        # Current accumulator
    findEquals = False            # True to find equals signs in arguments
    findPipe = False              # True to take notice of pipe characters
    noMoreGT = False              # True if there are no more greater-than (>) signs right of $i

    cnt = 0
    i = 0       # Input pointer, points to a pseudo-newline before start
    while True:
        cnt += 1
        if cnt > 100000:
            raise StandardError('Too long input. Aborting')

        # if fakeLineStart:
        #     found = 'line-start'
        #     curChar = ''
        # else:
        # Find next opening brace, closing brace or pipe
        search = searchBase
        # found = ''
        if stack.top is False:
            currentClosing = ''
        else:
            # print stack.top.data
            currentClosing = stack.top.data['close']
            search += currentClosing

        if findPipe:
            search += '|'
        if findEquals:
            # First equals will be for the template
            search += '='

        rule = None
        # Output literal section, advance input counter
        logger.debug('  [%4d] Advancing pointer until: %s', i,
                     search.replace('\n', '\\n'))
        literalLength = strcspn(text, search, i, debug=True)
        if literalLength > 0:
            stack.accum += htmlspecialchars(text[i:i + literalLength])
            logger.debug('  [%4d] Found literal section of length %d: "%s"',
                         i, literalLength, text[i:i + literalLength])
            i += literalLength

        if i >= len(text):
            # if currentClosing == "\n":
            #     # Do a past-the-end run to finish off the heading
            #     curChar = '';
            #     found = 'line-end';
            # else:
                # All done
                logger.debug('  [%4d] Reached the end (%d chars)',
                             i, len(text))
                break
        else:
            curChar = text[i]
            if curChar == '|':
                found = 'pipe'
            elif curChar == '=':
                found = 'equals'
            elif curChar == '<':
                found = 'angle'
            # elif curChar == r'\n':
            #     if inHeading:
            #         found = 'line-end'
            #     else:
            #         found = 'line-start'
            elif curChar == currentClosing:
                found = 'close'
            elif curChar in rules:
                found = 'open'
                rule = rules[curChar]
            else:
                i += 1
                continue

            if found == 'angle':
                matches = False

                # Determine element name
                matches = elementsRegex.match(text[i + 1:])
                if matches is None:
                    # Element name missing or not listed
                    logger.debug('  [%4d] Found tag with element name missing' +
                                 ' or not listed (excerpt: %s...)',
                                 i, text[i + 1:i + 1 + 10])
                    stack.accum += '&lt;'
                    i += 1
                    continue

                # Handle comments:
                # DM: We handle comments a bit differently than the original
                # PHP preprocessor. We just output them as they are, instead of
                # wrapping them in <comment> and htmlspecialchar-ize them.
                if matches.lastindex >= 2 and matches.group(2) == '!--':
                    # To avoid leaving blank lines, when a comment is both
                    # preceded and followed by a newline (ignoring spaces),
                    # trim leading and trailing spaces and one of the newlines.

                    # Find the end
                    endPos = text.find('-->', i + 4)
                    if endPos == -1:
                        # Unclosed comment in input, runs to end
                        inner = text[i:]
                        # Original code: stack.accum += '<comment>' +
                        # htmlspecialchars(inner) + '</comment>'
                        stack.accum += htmlspecialchars(inner)
                        i = lengthText
                    else:
                        startPos = i
                        endPos += 2
                        i = endPos + 1
                        inner = text[startPos:endPos + 1]
                        stack.accum += htmlspecialchars(inner)
                    continue

                name = matches.group(1)
                attrStart = i + len(name) + 1
                logger.debug('  [%4d] Found xml tag: %s', i, name)

                # Find end of tag
                tagEndPos = -1 if noMoreGT else text.find('>', attrStart)
                if tagEndPos == -1:
                    # Infinite backtrack
                    # Disable tag search to prevent worst-case O(N^2)
                    # performance
                    logger.debug('infinite backtrack')
                    noMoreGT = True
                    stack.accum += '&lt;'
                    i += 1
                    continue

                if text[tagEndPos - 1] == '/':
                    attrEnd = tagEndPos - 1
                    inner = ''
                    i = tagEndPos + 1
                    close = ''
                else:
                    attrEnd = tagEndPos
                    # Find closing tag
                    matches = re.search('<\/%s\s*>' % re.escape(name),
                                        text[tagEndPos + 1:])
                    if matches:
                        inner = text[tagEndPos + 1:tagEndPos + 1 + matches.start()]
                        i = tagEndPos + 1 + matches.start() \
                            + len(matches.group(0))
                        close = matches.group(0)
                    else:
                        # No end tag -- let it run out to the end of the text
                        logger.debug('         No end tag found')
                        inner = text[tagEndPos + 1:]
                        i = lengthText
                        close = ''

                if attrEnd <= attrStart:
                    attr = ''
                else:
                    attr = text[attrStart:attrEnd]

                tmp = (htmlspecialchars(name), htmlspecialchars(attr),
                       htmlspecialchars(inner), htmlspecialchars(close))
                stack.accum += '&lt;%s%s&gt;%s%s' % tmp

            elif found == 'open':
                # count opening brace characters
                count = strspn(text, curChar, i)
                # print "Found %d open braces" % count

                # we need to add to stack only if opening brace count
                # is enough for one of the rules
                if count >= rule['min']:

                    # Add it to the stack
                    piece = {
                        'open': curChar,
                        'close': rule['end'],
                        'count': count,
                        'lineStart': (i > 0 and text[i - 1] == r'\n')
                    }
                    stack.append(piece)
                    flags = stack.getFlags()
                    findPipe = flags['findPipe']
                    findEquals = flags['findEquals']

                else:
                    stack.accum += curChar * count

                i += count

            elif found == 'close':
                piece = stack.top
                # lets check if there are enough characters for closing brace
                maxCount = piece.data['count']
                count = strspn(text, curChar, i, maxCount)
                logger.debug('  [%4d] Found %d of %d close braces',
                             i, count, maxCount)

                # check for maximum matching characters (if there are 5
                # closing characters, we will probably need only
                # 3 - depending on the rules)
                rule = rules[piece.data['open']]
                if count > rule['max']:
                    # The specified maximum exists in the callback array,
                    # unless the caller has made an error
                    matchingCount = rule['max']
                else:
                    # Count is less than the maximum. Skip any gaps in the
                    # callback array to find the true largest match
                    # Need to use array_key_exists not isset because the
                    # callback can be null
                    matchingCount = count
                    while matchingCount > 0 \
                            and matchingCount not in rule['names'].keys():
                        matchingCount -= 1

                if matchingCount <= 0:
                    # No matching element found in callback array
                    # Output a literal closing brace and continue

                    # $accum .= htmlspecialchars(str_repeat($curChar,$count));
                    stack.accum += curChar * count
                    logger.debug('         ... which is not enough for' +
                                 ' current rule. Continuing...')

                    i += count
                    continue

                name = rule['names'][matchingCount]
                if name is None:
                    # No element, just literal text
                    element = piece.breakSyntax(matchingCount) \
                        + rule['end'] * matchingCount

                else:
                    # Create XML element
                    # Note: $parts is already XML, does not need to be encoded
                    parts = piece.parts
                    # print type(parts)
                    # logger.debug('%s', ', '.join([p for p in parts]))
                    title = parts[0].out
                    del parts[0]

                    # The invocation is at the start of the line if lineStart
                    # is set in the stack and all opening brackets are used up
                    if maxCount == matchingCount and piece.data['lineStart']:
                        attr = ' lineStart="1"'
                    else:
                        attr = ''

                    element = '<%s%s>' % (name, attr)
                    element += '<title>%s</title>' % title
                    logger.debug(' ' + element)

                    argIndex = 1
                    for part in parts:
                        if 'eqpos' in part:
                            argName = part.out[0:part.eqpos]
                            argValue = part.out[part.eqpos + 1:]
                            element += '<part><name>%s</name>=' % argName \
                                + '<value>%s</value></part>' % argValue
                        else:
                            element += '<part><name index="%s" />' % argIndex \
                                + '<value>%s</value></part>' % part.out
                            argIndex += 1
                    element += '</%s>' % name

                # Advance input pointer
                i += matchingCount

                # Unwind the stack
                stack.pop()

                # Re-add the old stack element if it still has unmatched
                # opening characters remaining
                if matchingCount < piece.data['count']:
                    piece.parts = [PPDPart()]
                    piece.data['count'] -= matchingCount
                    # do we still qualify for any callback with remaining cnt?
                    omin = rules[piece.data['open']]['min']
                    if piece.data['count'] >= omin:
                        stack.append(piece)
                    else:
                        stack.accum += piece.data['open'] * piece.data['count']

                flags = stack.getFlags()
                findPipe = flags['findPipe']
                findEquals = flags['findEquals']

                # Add XML element to the enclosing accumulator
                stack.accum += element

            elif found == 'pipe':
                findEquals = True    # shortcut for getFlags()
                stack.addPart()
                i += 1
            elif found == 'equals':
                findEquals = False   # shortcut for getFlags()
                stack.getCurrentPart().eqpos = len(stack.accum)
                stack.accum += '='
                i += 1

    # Output any remaining unclosed brackets
    for piece in stack.stack:
        logger.debug('Adding stack element with unclosed brackets')
        stack.rootAccum += piece.breakSyntax()

    stack.rootAccum += '</root>'
    xml = stack.rootAccum

    # print accum
    return xml

# xml = preprocessToXml('LLorem ipsum {{{{{Min lille mal}}} | Kake = sake }}')
# print xml



# encoding=utf-8
"""
mwtemplates
Copyright (c) 2012-2014 Dan Michael O. Heggø

Simple wikitext template parser and editor
"""

from __future__ import unicode_literals
from __future__ import print_function
import weakref
import six
# <root>LLorem ipsum <template>
# <title>
#     <tplarg>
#         <title>Min lille mal</title>
#     </tplarg>
# </title>
# <part>
#     <name index="1"/>
#     <value>sorona</value>
# </part>
# <part>
#     <name>Kake</name>=<value>sake</value>
# </part>
# </template>
# </root>


# <root>LLorem ipsum
# <template>
#   <title>
#       <tplarg>
#           <title>
#               Min
#               <template>
#                   <title>lille</title>
#               </template>
#               mal
#           </title>
#       </tplarg>
#       son
#   </title>
# <part>
#     <name index="1"/>
#     <value>sorona</value>
# </part>
# <part>
#     <name>Kake</name>=<value>sake</value>
# </part>
# </template>
# </root>

import logging
import re
from lxml import etree
try:
    from io import StringIO  # Python 3
except ImportError:
    from StringIO import StringIO  # Python 2
from mwtemplates.preprocessor import preprocessToXml

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


class TemplateParseError(Exception):
    pass


def most_common(lst):
    return max(set(lst), key=lst.count)


def get_whitespace(txt):
    """
    Returns a list containing the whitespace to the left and
    right of a string as its two elements
    """

    # if the entire parameter is whitespace
    rall = re.search(r'^([\s])+$', txt)
    if rall:
        tmp = txt.split('\n', 1)
        if len(tmp) == 2:
            return (tmp[0], '\n' + tmp[1])  # left, right
        else:
            return ('', tmp[0])  # left, right
    left = ''
    # find whitespace to the left of the parameter
    rlm = re.search(r'^([\s])+', txt)
    if rlm:
        left = rlm.group(0)
    right = ''
    # find whitespace to the right of the parameter
    rrm = re.search(r'([\s])+$', txt)
    if rrm:
        right = rrm.group(0)
    return (left, right)

# try:
#     self.doc = etree.parse(StringIO(docstr), parser)
#     #self.doc = etree.parse(self.filename, parser)
#     for e in parser.error_log:
#         print "Warning: "+e.message
# except etree.XMLSyntaxError:
#     print "Failed to parse xml file: ",filename
#     for e in parser.error_log:
#         print "Warning: "+e.message
#     sys.exit(2)


def get_wikitext(node):
    tmp = ''
    if node.text is not None:
        tmp += node.text
    for child in node:
        if child.tag == 'tplarg':
            tmp += '{{{%s}}}' % get_wikitext(child)
        elif child.tag == 'template':
            tmp += '{{%s}}' % get_wikitext(child)
        elif child.tag in ['title', 'name', 'value']:
            tmp += get_wikitext(child)
        elif child.tag == 'part':
            tmp += '|' + get_wikitext(child)
        else:
            raise TemplateParseError("Found unknown tag: %s"
                                     % etree.tostring(child))
        if child.tail is not None:
            tmp += child.tail
    return tmp


# def get_arg(node):
#    node.xpath('name')[0]
#    node.xpath('value')[0]


class Templates(object):
    """
    Dictionary-like class to hold templates.
    The first character of the key is case-insensitive, to mimic MediaWiki,
    where {{Hello}} and {{hello}} are considered the same template, while
    {{Hello}} and {{HEllo}} are not.
    """

    def __init__(self, editor, doc):

        # Make child-parent reference weak to avoid memory leak
        self.editor = weakref.ref(editor)

        self.doc = doc
        # self._entries = self._templates()

    def _templates(self):
        templates = []
        for node in self.doc.findall('.//template'):
            q = Template(node, self.editor)
            if q.key != '':
                templates.append(q)
        return templates

    # def add(self, node):
    #     self._entries.append(Template(node, self.editor))

    def __contains__(self, tpl_name):
        tpl_name = tpl_name[0].upper() + tpl_name[1:]
        for tpl in self._templates():
            if tpl.key == tpl_name:
                return True
        return False

    def __getitem__(self, tpl_name):
        tpl_name = tpl_name[0].upper() + tpl_name[1:]
        return [tpl for tpl in self._templates() if tpl.key == tpl_name]
        # raise KeyError

    def __repr__(self):
        return self._templates().__repr__()

    def __len__(self):
        return len(self._templates())

    def keys(self):
        return list(set([tpl.key for tpl in self._templates()]))

    def items(self):
        return [(key, self[key]) for key in self.keys()]

    def iteritems(self):
        for key in self.keys():
            yield (key, self[key])

    # def __contains__(self, x):
    #     return self.get_key(x) in self._entries

    # def __getitem__(self, x):
    #     return self._entries[self.get_key(x)]

    # def __setitem__(self, x, val):
    #     self._entries[self.get_key(x)] = val

    # def values():
    #     return self._entries.values()

    # def items(self):
    #     return self._entries.items()


class Parameters(object):
    """
    Dictionary-like class to hold a template's parameters.
    """

    def __init__(self, template):
        self._entries = []
        self.template = template

    def __contains__(self, param):
        # param is a Parameter object or a string
        if param in self._entries:
            return True
        return param in self.keys()

    def __getitem__(self, param_name):
        for param in self._entries:
            if param.key == param_name:
                return param
        raise KeyError('No such parameter')

    def get(self, param_name, default=None):
        try:
            return self.__getitem__(param_name)
        except KeyError:
            return default

    def __setitem__(self, name, val):

        if isinstance(val, six.integer_types):
            val = six.text_type(val)
        elif not isinstance(val, six.string_types):
            raise TypeError

        # Check if parameter exists
        for entry in self._entries:
            if entry.key == name:
                # parameter changed:
                entry.edit(val)
                return

        # If not, let's add it, trying to use any existing whitespace pattern:
        name_ws, value_ws = self.find_whitespace_pattern()
        # print '"' + whitespace[0] + '", "' + whitespace[1] + '"'
        name = name_ws[0] + name + name_ws[1]
        val = value_ws[0] + val + value_ws[1]
        parentnode = self.template.node
        node = etree.XML('<part><name>%s</name>=<value>%s</value></part>'
                         % (name, val))
        parentnode.append(node)
        self.add(node)

    def __delitem__(self, param_name):
        param = self.__getitem__(param_name)
        return self.remove(param)

    def remove(self, param):
        # param can be a Parameter object or a string

        if param in self.keys():
            param = self.__getitem__(param)

        i = self.index(param)
        logger.debug('Removing parameter "%s"', param.key)
        self.template.node.remove(param.node)
        del self._entries[i]

    def __repr__(self):
        return '<Parameters: %s>' % (', '.join(['%s="%s"' % (x.key, x.value) for x in self._entries]))

    def keys(self):
        # Return all parameter keys, including duplicates
        return [param.key for param in self._entries]

    def index(self, param):
        return self._entries.index(param)

    def __len__(self):
        return len(self._entries)

    def __iter__(self):
        # Iterate over all parameters, including duplicates
        for param in self._entries:
            yield param

    def items(self):
        # All parameters, including duplicates
        return [(param.key, param) for param in self._entries]

    def iteritems(self):
        # Iterate over all parameters, including duplicates
        for param in self._entries:
            yield (param.key, param)

    def add(self, node):
        self._entries.append(Parameter(node))

    def find_whitespace_pattern(self):
        """
        Try to find a whitespace pattern in the existing parameters
        to be applied to a newly added parameter
        """
        name_ws = []
        value_ws = []
        for entry in self._entries:
            name_ws.append(get_whitespace(entry.name))
            if entry.value != '':
                value_ws.append(get_whitespace(entry._value))  # _value is unstripped
        if len(value_ws) >= 1:
            value_ws = most_common(value_ws)
        else:
            value_ws = ('', ' ')
        if len(name_ws) >= 1:
            name_ws = most_common(name_ws)
        else:
            name_ws = (' ', '')
        return name_ws, value_ws


class Parameter(object):
    """
    Class to hold a single template parameter.
    """

    def __init__(self, node):
        self.node = node
        namenode = node.xpath('name')[0]
        self._name = get_wikitext(namenode)
        self._value = get_wikitext(node.xpath('value')[0])
        idx = namenode.get('index')
        if idx:
            self._index = int(idx)
        else:
            self._index = -1
        logger.debug('  added parameter: %s=%s',
                     self._name.strip(), self._value.strip())

    def edit(self, val):
        if isinstance(val, six.integer_types):
            val = six.text_type(val)
        elif not isinstance(val, six.string_types):
            raise TypeError

        logger.debug('Parameter "%s" changed: "%s" -> "%s"',
                     self.key, self.value.strip(), val)
        whitespace = get_whitespace(self._value)
        self._value = whitespace[0] + val + whitespace[1]
        # self.node.xpath('value')[0].text = self._value
        self.node.replace(self.node.xpath('value')[0],
                          etree.XML('<value>' + self._value + '</value>'))

    # @property
    # def index(self):
    #    return self._index

    @property
    def name(self):
        return self._name

    @property
    def key(self):
        if self._index != -1:
            return self._index
        else:
            tmp = self.name.strip()
            if tmp.isnumeric():
                return int(tmp)
            else:
                return tmp

    @property
    def value(self):
        return self._value.strip()

    @value.setter
    def value(self, val):
        self.edit(val)

    def __eq__(self, param):
        if type(param).__name__ == 'unicode':
            return self.__unicode__() == param
        elif type(param).__name__ == 'str':
            return self.__str__() == param
        elif type(param).__name__ == 'int':
            try:
                return int(self.__unicode__()) == param
            except ValueError:
                return False
        else:
            return False

    def __ne__(self, param):
        return not self.__eq__(param)

    def __unicode__(self):
        return self._value.strip()

    def __str__(self):
        if six.PY3:
            return self.__unicode__()
        else:
            return self.__unicode__().encode('utf-8')

    def __float__(self):
        return float(self.__unicode__())

    def __int__(self):
        return int(self.__unicode__())

    def __repr__(self):
        return self.value


class Template(object):
    """
    Class to hold a single template.
    """

    def __init__(self, node, editor):

        logger.debug('Initiating Template object')

        self.node = node
        self.editor = editor
        self._name = ''
        self.parameters = Parameters(self)
        for elem in node:
            if elem.tag == 'title':
                self._name = get_wikitext(elem)
                logger.debug('  name: %s', self.name)
            elif elem.tag == 'part':
                self.parameters.add(elem)

    def __repr__(self):
        return '<Template:"%s" at line %d>' % (self.key, self.node.sourceline)

    @property
    def key(self):
        tmp = self._name.strip()
        tmp = re.sub(r'^(?:[Mm]al|[Tt]emplate):', '', tmp)
        if len(tmp) == 0:
            return ''
        return tmp[0].upper() + tmp[1:]

    @property
    def name(self):
        return self.key

    @name.setter
    def name(self, val):
        whitespace = get_whitespace(self._name)
        self._name = whitespace[0] + val + whitespace[1]
        self.node.xpath('title')[0].text = self._name

    def has_param(self, name):
        """ Returns True if the parameter is defined and non-empty """
        return name in self.parameters and self.parameters[name] != ''

    def get_anonymous_parameters(self):
        """
        Returns the anonymous (non-named) parameters as a normal list, but with
        indices starting on 1, not 0, to comply with standard mediawiki practice.
        """
        tmp = {}
        for param in self.parameters:
            if type(param.key) == int:
                tmp[param.key] = param.value
        lst = [None]
        for param in range(1, max(tmp.keys()) + 1):
            lst.append(tmp[param])
        return lst

    def __unicode__(self):
        tmp = '{{%s' % self.name
        for param in self.parameters:
            tmp += '\n |%s=%s' % (param.key, param.value)
        if len(self.parameters) > 0:
            tmp += '\n'
        tmp += '}}'

        return tmp

    def __str__(self):
        if six.PY3:
            return self.__unicode__()
        else:
            return self.__unicode__().encode('utf-8')

    def remove(self):
        logger.debug('Removing node "%s"', self.key)
        parent = self.node.getparent()
        prevnode = None
        for node in parent.getchildren():
            if node == self.node:
                if prevnode is None:
                    if parent.text is None:
                        parent.text = self.node.tail.lstrip()
                    else:
                        parent.text += self.node.tail.lstrip()

                else:
                    if prevnode.tail is None:
                        prevnode.tail = self.node.tail.lstrip()
                    else:
                        prevnode.tail += self.node.tail.lstrip()
                parent.remove(self.node)
                return
            else:
                prevnode = node


class TemplateEditor(object):
    r"""
    TemplateEditor is the main class to work with templates in some wikitext.

    Example:

    >>> wikitext = '{{infobox country\n' \
    ...    + '|name=Fantasia\n' \
    ...    + '|population_census=4,830,300 }}'
    >>> editor = TemplateEditor(wikitext)
    >>> tpl = editor.templates['Infobox country'][0]
    >>> tpl.parameters['population_census'] = '5,033,676'
    >>> print(editor.wikitext())
    {{infobox country
    |name=Fantasia
    |population_census=5,033,676 }}
    """

    def __init__(self, text):
        xml = preprocessToXml(text)
        parser = etree.XMLParser()
        self.text = text
        self.doc = etree.parse(StringIO(xml), parser)
        # root = self.doc.xpath('/root')[0]

        self.templates = Templates(self, self.doc)

        # self.scan_for_templates()

    def xml(self):
        # for key, val in self.templates.items():
        #     for tpl in val:
        #         tpl.save()
        return etree.tostring(self.doc, encoding='unicode')

    def wikitext(self):
        # for key, val in self.templates.items():
        #     for tpl in val:
        #         tpl.save()
        txt = get_wikitext(self.doc.xpath('/root')[0])
        return txt



#!/usr/bin/env python
# encoding=utf-8
from __future__ import print_function

import os
import sys

try:
    from setuptools import setup
    from setuptools.command.test import test as TestCommand
except ImportError:
    print("This package requires 'setuptools' to be installed.")
    sys.exit(1)


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = '-v --pep8 tests mwtemplates'

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # Import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

setup(name='mwtemplates',
      version='0.3.4',
      description='MediaWiki template parser and editor',
      long_description=README,
      classifiers=[
          'Topic :: Text Processing :: Markup',
          'License :: OSI Approved :: MIT License',
          'Intended Audience :: Developers',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
      ],
      keywords='mediawiki wikipedia',
      author='Dan Michael Heggø',
      author_email='danmichaelo@gmail.com',
      url='https://github.com/danmichaelo/mwtemplates',
      license='MIT',
      packages=['mwtemplates'],
      install_requires=['lxml', 'six'],
      cmdclass={'test': PyTest},
      tests_require=['pytest-pep8', 'pytest-cache', 'pytest', 'mock']
      )
